﻿

////////--------------- START: OCEAN -----------------
#version 330 core

//mainly http://http.developer.nvidia.com/GPUGems2/gpugems2_chapter19.html 

in vec4 waveSize0;
in vec2 waveSize1;
in vec2 waveSize2;
in vec2 waveSize3;
in vec2 waveSize4;

in vec4 screenPos;
in vec3 eyePos;

uniform sampler2D normalMap;		
uniform sampler2D refracTex;	//in our case bottom of skybox	
uniform sampler2D reflectionTex;	


out vec4 color;


void main()
{
	vec3 vEye = normalize(eyePos);
	vec2 vProj = screenPos.xy / screenPos.w;

	vec3 waveTex0 = texture2D(normalMap, waveSize0.xy).xyz;
	vec3 waveTex1 = texture2D(normalMap, waveSize1.xy).xyz;
	vec3 waveTex2 = texture2D(normalMap, waveSize2.xy).xyz;
	vec3 waveTex3 = texture2D(normalMap, waveSize3.xy).xyz;
	vec3 waveTex4 = texture2D(normalMap, waveSize4.xy).xyz;

	vec3 avWaveTex = normalize(2.0 * (waveTex0.xyz + waveTex1.xyz + waveTex2.xyz + waveTex3.xyz + waveTex4.xyz) - 5.0);

	//refraction tex
	vec3 waveRefract = avWaveTex * vec3(0.03, 0.03, 1.0);
	vec2 refractCoord = vProj + waveRefract.xy;
	vec3 vRefraction = texture2D(refracTex, vec2(refractCoord.x, 1 - refractCoord.y)).xyz;

	//reflection tex
	vec3 waveReflect = avWaveTex * vec3(0.05, 0.05, 1.0);
	vec2 reflectCoord = vProj + waveReflect.xy;
	vec3 vReflection = texture2D(reflectionTex, reflectCoord).rgb;


	// Fresnel
	float NdotL = max(dot(vEye, normalize(waveReflect)), 0.0);
	float facing = (1.0 - NdotL);
	const float r0 = 0.1; 
	float frPow = 5.0;
	//schlick
	float fresnel = max(r0 + (1.0 - r0) * pow((1.0 - NdotL), frPow), 0.0);


	//water colour
	float dist = waveSize0.w;
	dist = min(dist, 30);
	float fDistScale = clamp(20.0 / dist, 0.0, 1.0);
	vec3 col = vec3( 0.1f, 0.5f, 0.7f)/100; //set colour, then weaken


	vec3 waterColor = (col * facing + (vRefraction * fDistScale + (1 - fDistScale) * col) * (1.0 - facing)); //colour based on distance/depth
	color = vec4(fresnel * vReflection + waterColor * (1 - fresnel), 1.0);

}
////--------------- END: OCEAN -----------------







//////--------------- START: MIRROR -----------------
//#version 330 core


//uniform mat4 reflectView;
//uniform sampler2D reflectionTex;

//in vec3 interpolate;

//out vec4  color;

//void main()
//{
//	vec4 vClipReflection = reflectView * vec4(interpolate.xy, 0.0 , 1.0);
//	vec2 vDeviceReflection = vClipReflection.st / vClipReflection.q;
//	vec2 vTextureReflection = vec2(0.5, 0.5) + 0.5 * vDeviceReflection;

//	vec4 reflectionTextureColor = texture2D (reflectionTex, vTextureReflection);

//	// Framebuffer reflection can have alpha > 1
//	reflectionTextureColor.a = 1.0;

//	//color = vec4(0.5f, 0.0f, 0.0f, 1.0f);

//	color = reflectionTextureColor;
//}

////--------------- END:  MIRROR -----------------




//////--------------- START: SIMPLE -----------------

//#version 330 core

//in vec2 TexCoords;
//in vec3 normal;

//out vec4 color;

//uniform sampler2D refracTex;

//void main()
//{    
//   color = vec4(texture(refracTex, TexCoords));
//	//color = vec4(0.5f, 0.0f, 0.0f, 1.0f);
//	// color = vec4(normal, 1.0f);
//}
//////--------------- END: SIMPLE -----------------
